--[[
Author: Starinnia
Helper mod for the Thaddius encounter in Naxxramas
Project version: r9
$Date: 2008-12-22 03:22:00 +0000 (Mon, 22 Dec 2008) $
$Revision: 16 $

Copyright (c) 2007-2008 Michael J. Murray aka Starinnia of Lothar

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must not
claim that you wrote the original software. If you use this software
in a product, an acknowledgment in the product documentation would be
appreciated but is not required.

2. Altered source versions must be plainly marked as such, and must not be
misrepresented as being the original software.

3. This notice may not be removed or altered from any source
distribution.
]]

local ChargeMeter = LibStub("AceAddon-3.0"):NewAddon("ChargeMeter", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0", "LibSink-2.0")
local LSM = LibStub("LibSharedMedia-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("ChargeMeter")
_G["ChargeMeter"] = ChargeMeter

local posTex = "Interface\\Icons\\Spell_ChargePositive"
local negTex = "Interface\\Icons\\Spell_ChargeNegative"
local warning = L["RUN! You have a new charge!"]
local colorInfo = {["r"] = 1, ["g"] = 0, ["b"] = 0}
local newCharge
local currentCharge
local timer
local fadeTimer
local frame
local frameTest

--register some sounds since we can't depend on the user having Omen or BigWigs
--both of which give some nice warning sounds
LSM:Register("sound", "BigWigs: Info", [[Interface\AddOns\ChargeMeter\Sound\Info.mp3]])
LSM:Register("sound", "BigWigs: Alert", [[Interface\AddOns\ChargeMeter\Sound\Alert.mp3]])
LSM:Register("sound", "BigWigs: Alarm", [[Interface\AddOns\ChargeMeter\Sound\Alarm.mp3]])
LSM:Register("sound", "Rubber Ducky", [[Sound\Doodad\Goblin_Lottery_Open01.wav]])
LSM:Register("sound", "Cartoon FX", [[Sound\Doodad\Goblin_Lottery_Open03.wav]])
LSM:Register("sound", "Explosion", [[Sound\Doodad\Hellfire_Raid_FX_Explosion05.wav]])
LSM:Register("sound", "Shing!", [[Sound\Doodad\PortcullisActive_Closed.wav]])
LSM:Register("sound", "Wham!", [[Sound\Doodad\PVP_Lordaeron_Door_Open.wav]])
LSM:Register("sound", "Simon Chime", [[Sound\Doodad\SimonGame_LargeBlueTree.wav]])
LSM:Register("sound", "War Drums", [[Sound\Event Sounds\Event_wardrum_ogre.wav]])
LSM:Register("sound", "Cheer", [[Sound\Event Sounds\OgreEventCheerUnique.wav]])
LSM:Register("sound", "Humm", [[Sound\Spells\SimonGame_Visual_GameStart.wav]])
LSM:Register("sound", "Short Circuit", [[Sound\Spells\SimonGame_Visual_BadPress.wav]])
LSM:Register("sound", "Fel Portal", [[Sound\Spells\Sunwell_Fel_PortalStand.wav]])
LSM:Register("sound", "Fel Nova", [[Sound\Spells\SeepingGaseous_Fel_Nova.wav]])
LSM:Register("sound", "Raid Warning", [[Sound\Interface\RaidWarning.wav]])
LSM:Register("sound", "Lazer", [[Interface\AddOns\ChargeMeter\Sound\Lazer.mp3]])
LSM:Register("sound", "Breaking Glass", [[Interface\AddOns\ChargeMeter\Sound\Breaking_Glass.mp3]])

local CHATFRAMES = {
	["ChatFrame1"] = string.format(L["Chat Frame %d"], 1),
	["ChatFrame3"] = string.format(L["Chat Frame %d"], 3),
	["ChatFrame4"] = string.format(L["Chat Frame %d"], 4),
	["ChatFrame5"] = string.format(L["Chat Frame %d"], 5),
	["ChatFrame6"] = string.format(L["Chat Frame %d"], 6),
	["ChatFrame7"] = string.format(L["Chat Frame %d"], 7),
}

local defaults = {
	profile = {
		icon = false,
		raid_warn = true,
		flash = true,
		shake = true,
		combattext = true,
		sound = false,
		soundChoice = "None",
		chatframes = false,
		frames = {
			["ChatFrame1"] = true,
			["ChatFrame3"] = false,
			["ChatFrame4"] = false,
			["ChatFrame5"] = false,
			["ChatFrame6"] = false,
			["ChatFrame7"] = false,
		},
		iconOpts = {
			size = 20,
			scale = 1.0,
			alpha = 0.5,
			x = 0,
			y = 0,
			duration = 0,
		},
		sinkOpts = {},
	},
}

local opts
local function getOpts()
	if not opts then
		opts = {
			type = 'group',
			childGroups = 'tab',
			args = {
				main = {
					type = 'group',
					name = L["Main"],
					args = {
						general = {
							type = 'group',
							name = L["General"],
							order = 1,
							args = {
								icon = {
									type = 'toggle',
									name = L["Charge Icon"],
									get = function() return ChargeMeter.db.profile.icon end,
									set = function(_, v)
										ChargeMeter.db.profile.icon = v
										if v then
											if not frame then
												frame = ChargeMeter:CreateChargeIcon()
											end
										end
									end,
									order = 1,
								},
								raidwarn = {
									type = 'toggle',
									name = L["Raid Warning"],
									desc = L["Put a warning in the raid warning frame (only you see this)."],
									get = function() return ChargeMeter.db.profile.raid_warn end,
									set = function() ChargeMeter.db.profile.raid_warn = not ChargeMeter.db.profile.raid_warn end,
									order = 2,
								},
								flash = {
									type = 'toggle',
									name = L["Flash Effect"],
									get = function() return ChargeMeter.db.profile.flash end,
									set = function() ChargeMeter.db.profile.flash = not ChargeMeter.db.profile.flash end,
									order = 3,
								},
								shake = {
									type = 'toggle',
									name = L["Shake Effect"],
									get = function() return ChargeMeter.db.profile.shake end,
									set = function() ChargeMeter.db.profile.shake = not ChargeMeter.db.profile.shake end,
									order = 4,
								},
								testfns = {
									type = 'execute',
									name = L["Test Flash 'n Shake"],
									func = function()
										if ChargeMeter.db.profile.flash then ChargeMeter:Flash() end
										if ChargeMeter.db.profile.shake then ChargeMeter:Shake() end
									end,
									order = 5,
								},
								combattext = {
									type = 'toggle',
									name = L["Send to Combat Text"],
									get = function() return ChargeMeter.db.profile.combattext end,
									set = function() ChargeMeter.db.profile.combattext = not ChargeMeter.db.profile.combattext end,
									order = 6,
								},
								sound = {
									type = 'toggle',
									name = L["Sound Effect"],
									get = function() return ChargeMeter.db.profile.sound end,
									set = function() ChargeMeter.db.profile.sound = not ChargeMeter.db.profile.sound end,
									order = 7,
								},
								selectsound = {
									type = 'select',
									name = L["Choose Sound Effect"],
									dialogControl = "LSM30_Sound" ,
									get = function() return ChargeMeter.db.profile.soundChoice end,
									set = function(_, k) ChargeMeter.db.profile.soundChoice = k end,
									values = AceGUIWidgetLSMlists.sound,
									order = 8,
								},
								chatframes = {
									type = 'multiselect',
									name = L["Chat Frames"],
									desc = L["Choose which chat frames will receive output."],
									get = function(_, k) return ChargeMeter.db.profile.frames[k] end,
									set = function(_, k, v) ChargeMeter.db.profile.frames[k] = v end,
									values = CHATFRAMES,
									order = 9,
								},
							},
						},
						warnings = {
							type = 'group',
							name = L["Warning Messages"],
							order = 3,
							args = {
								raidwarn = {
									type = 'input',
									name = L["Raid Warning Message"],
									get = function() return ChargeMeter.db.profile.rwMessage or warning end,
									set = function(_, v) ChargeMeter.db.profile.rwMessage = v end,
									order = 1,
								},
								combattext = {
									type = 'input',
									name = L["Combat Test Message"],
									get = function() return ChargeMeter.db.profile.ctMessage or warning end,
									set = function(_, v) ChargeMeter.db.profile.ctMessage = v end,
									order = 2,
								},
								chatframe = {
									type = 'input',
									name = L["Chat Frame Message"],
									get = function() return ChargeMeter.db.profile.cfMessage or warning end,
									set = function(_, v) ChargeMeter.db.profile.cfMessage = v end,
									order = 3,
								},
							},
						},
						icon = {
							type = 'group',
							name = L["Charge Icon Options"],
							hidden = function()
								if ChargeMeter.db.profile.icon then
									return false
								else
									return true
								end
							end,
							order = 2,
							args = {
								size = {
									type = 'range',
									name = L["Icon Size"],
									desc = L["Set the size of the icon. Note: Icon is always square"],
									get = function() return ChargeMeter.db.profile.iconOpts.size end,
									set = function(_, v)
										ChargeMeter.db.profile.iconOpts.size = v
										if frame then
											frame:SetHeight(v)
											frame:SetWidth(v)
										end
									end,
									min = 5,
									max = 350,
									step = 5,
									order = 6,
								},
								scale = {
									type = 'range',
									name = L["Icon Scale"],
									get = function() return ChargeMeter.db.profile.iconOpts.scale end,
									set = function(_, v)
										ChargeMeter.db.profile.iconOpts.scale = v
										if frame then
											frame:SetScale(v)
										end
									end,
									min = 0.1,
									max = 2.5,
									step = 0.1,
									order = 6
								},
								x = {
									type = 'input',
									name = L["X Position"],
									get = function() return frame and tostring(ChargeMeter.db.profile.iconOpts.x / frame:GetEffectiveScale()) or 0 end,
									set = function(_, v)
										v = tonumber(v)
										local s = block:GetEffectiveScale()
										ChargeMeter.db.profile.iconOpts.x = v * s
										if frame then
											frame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", v, ChargeMeter.db.profile.iconOpts.y / s)
										end
									end,
									order = 4,
								},
								y = {
									type = 'input',
									name = L["Y Position"],
									get = function() return frame and tostring(ChargeMeter.db.profile.iconOpts.y / frame:GetEffectiveScale()) or 0 end,
									set = function(_, v)
										v = tonumber(v)
										local s = block:GetEffectiveScale()
										ChargeMeter.db.profile.iconOpts.y = v * s
										if frame then
											frame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", ChargeMeter.db.profile.iconOpts.x / s, v)
										end
									end,
									order = 5,
								},
								alpha = {
									type = 'range',
									name = L["Icon Alpha"],
									get = function() return ChargeMeter.db.profile.iconOpts.alpha end,
									set = function(_, v)
										ChargeMeter.db.profile.iconOpts.alpha = v
										if frame then
											frame:SetAlpha(v)
										end
									end,
									min = 0.01,
									max = 1,
									step = 0.01,
									order = 3,
								},
								duration = {
									type = 'range',
									name = L["Icon Duration"],
									desc = L["Set how long the icon will display itself after a polarity shift. A value of zero will always show the icon."],
									get = function() return ChargeMeter.db.profile.iconOpts.duration end,
									set = function(_, v) ChargeMeter.db.profile.iconOpts.duration = v end,
									min = 0,
									max = 15,
									step = 1,
									order = 2,
								},
								test = {
									type = 'execute',
									name = L["Force Show"],
									desc = L["Force addon to display the icon to view changes. Click again to hide icon."],
									func = function()
										if frameTest then
											frame:Hide()
											frame:EnableMouse(false)
											frameTest = nil
										else
											if frame then
												frame:Show()
											else
												frame = ChargeMeter:CreateChargeIcon()
												frame:Show()
											end
											frame:EnableMouse(true)
											frameTest = true
										end
									end,
									order = 1,
								},
							},
						},
					},
				},
				output = ChargeMeter:GetSinkAce3OptionsDataTable(),
				profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(ChargeMeter.db),
			},
		}
	end
	return opts
end

function ChargeMeter:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("ChargeMeterDB", defaults, "Default")
	self.db.RegisterCallback(self, "OnProfileReset", "ResetDB")
	self.db.RegisterCallback(self, "OnProfileCopied", "RefreshDB")
	self.db.RegisterCallback(self, "OnProfileChanged", "RefreshDB")
	self:SetSinkStorage(self.db.profile.sinkOpts)
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("ChargeMeter", getOpts)
	self:RegisterChatCommand("chargemeter", "OpenConfig", true, true)
	self:RegisterChatCommand("cmeter", "OpenConfig")
end

function ChargeMeter:OnEnable()
	self:RegisterEvent("CHAT_MSG_MONSTER_YELL", "BossYell")
end

function ChargeMeter:ResetDB()
	if frame then
		local db = self.db.profile.iconOpts
		frame:ClearAllPoints()
		frame:SetPoint("CENTER", UIParent, "CENTER")
		frame:SetWidth(db.size)
		frame:SetHeight(db.size)
		frame:SetScale(db.scale)
		frame:SetAlpha(db.alpha)
	end
end

function ChargeMeter:RefreshDB()
	if frame then
		local db = self.db.profile.iconOpts
		local s = frame:GetEffectiveScale()
		frame:ClearAllPoints()
		frame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", db.x / s, db.y / s)
		frame:SetWidth(db.size)
		frame:SetHeight(db.size)
		frame:SetScale(db.scale)
		frame:SetAlpha(db.alpha)
	end
end

function ChargeMeter:OpenConfig()
	LibStub("AceConfigDialog-3.0"):SetDefaultSize("ChargeMeter", 500, 600)
	LibStub("AceConfigDialog-3.0"):Open("ChargeMeter")
end

function ChargeMeter:BossYell(msg)
	if msg == L["Eat... your... bones..."] or msg == L["Break... you!!"] or msg == L["Kill..."] then
		self:Print("ChargeMeter activated.")
		timer = self:ScheduleRepeatingTimer("ChargeCheck", 3)
		self:RegisterEvent("PLAYER_REGEN_ENABLED", "ResetMeter")
		if self.db.profile.icon then
			if not frame then
				frame = self:CreateChargeIcon()
			end
		end
	end
end

function ChargeMeter:ResetMeter()
	self:Print("ChargeMeter deactivated.")
	self:UnregisterEvent("PLAYER_REGEN_ENABLED")
	self:CancelTimer(timer, true)
	timer = nil
	newCharge, currentCharge = nil, nil
end

function ChargeMeter:IconHide()
	self:CancelTimer(fadeTimer, true)
	fadeTimer = nil
	frame:Hide()
end

function ChargeMeter:ChargeCheck()
	local i = 1
	while UnitDebuff("player", i) do
		local _, _, icon, count = UnitDebuff("player", i)
		if icon == negTex then
			if count > 1 then return end
			newCharge = "negative"
		elseif icon == posTex then
			if count > 1 then return end
			newCharge = "positive"
		end
		i = i + 1
	end
	
	if newCharge then
		if currentCharge == newCharge then
			--no change
			self:Print("No charge change")
		else
			--move yer ass
			self:Print("New Charge, move it!")
			
			--update the icon
			if self.db.profile.icon then
				if newCharge == "positive" then
					frame.tex:SetTexture(posTex)
				else
					frame.tex:SetTexture(negTex)
				end
				if not frame:IsVisible() then
					frame:Show()
					if self.db.profile.iconOpts.duration > 0 then
						fadeTimer = self:ScheduleTimer("IconHide", self.db.profile.iconOpts.duration)
					end
				end
			end
			
			--send out warning texts
			if self.db.profile.raid_warn then
				RaidNotice_AddMessage(RaidWarningFrame, self.db.profile.rwMessage or warning, colorInfo)
			end
			if self.db.profile.combattext then
				self:Pour(self.db.profile.ctMessage or warning, 1, 0, 0)
			end
			if self.db.profile.chatframes then
				for k, v in pairs(self.db.profile.frames) do
					if v then
						_G[k]:AddMessage(self.db.profile.cfMessage or warning)
					end
				end
			end
			
			--play sound
			if self.db.profile.sound then
				PlaySoundFile(LSM:Fetch("sound", self.db.profile.soundChoice))
			end
			
			--flash 'n shake
			if self.db.profile.flash then self:Flash() end
			if self.db.profile.shake then self:Shake() end
		end
		currentCharge = newCharge
	end
	newCharge = nil
end

function ChargeMeter:CreateChargeIcon()
	local db = self.db.profile.iconOpts
	local f = CreateFrame("FRAME", "ChargeMeterChargeIcon")
	f.tex = f:CreateTexture()
	f.tex:ClearAllPoints()
	f.tex:SetAllPoints(f)
	f.tex:SetTexture(posTex)
	f:SetWidth(db.size)
	f:SetHeight(db.size)
	f:SetScale(db.scale)
	f:SetAlpha(db.alpha)
	
	if db.x == 0 and db.y == 0 then
		f:ClearAllPoints()
		f:SetPoint("CENTER", UIParent, "CENTER")
	else
		local s = f:GetEffectiveScale()
		f:ClearAllPoints()
		f:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", db.x / s, db.y / s)
	end
	
	f:RegisterForDrag("LeftButton")
	f:EnableMouse(false)
	f:SetMovable(true)
	f:SetClampedToScreen(true)
	f:SetScript("OnDragStart", function(self) self:StartMoving() end)
	f:SetScript("OnDragStop", function(self)
		self:StopMovingOrSizing()
		local s = self:GetEffectiveScale()
		db.x = self:GetLeft() * s
		db.y = self:GetTop() * s
	end)
	
	f:Hide()
	
	return f
end

--Flash and Shake effects are taken from Omen3
--and are originally from BigWigs
function ChargeMeter:Flash()
	if not self.FlashFrame then
		local flasher = CreateFrame("FRAME", "ChargeMeterFlashFrame")
		flasher:SetToplevel(true)
		flasher:SetFrameStrata("FULLSCREEN_DIALOG")
		flasher:SetAllPoints(UIParent)
		flasher:EnableMouse(false)
		flasher:Hide()
		flasher.texture = flasher:CreateTexture(nil, "BACKGROUND")
		flasher.texture:SetTexture("Interface\\FullScreenTextures\\OutOfControl")
		flasher.texture:SetAllPoints(UIParent)
		flasher.texture:SetBlendMode("ADD")
		flasher:SetScript("OnShow", function(self)
			if currentCharge == "positive" then
				self.texture:SetTexture("Interface\\FullScreenTextures\\OutOfControl")
			else
				self.texture:SetTexture("Interface\\FullScreenTextures\\LowHealth")
			end
			self.elapsed = 0
			self:SetAlpha(0)
		end)
		flasher:SetScript("OnUpdate", function(self, elapsed)
			elapsed = self.elapsed + elapsed
			if elapsed < 2.6 then
				local alpha = elapsed % 1.3
				if alpha < 0.15 then
					self:SetAlpha(alpha / 0.15)
				elseif alpha < 0.9 then
					self:SetAlpha(1 - (alpha - 0.15) / 0.6)
				else
					self:SetAlpha(0)
				end
			else
				self:Hide()
			end
			self.elapsed = elapsed
		end)
		self.FlashFrame = flasher
	end

	self.FlashFrame:Show()
end

function ChargeMeter:Shake()
	local shaker = self.ShakerFrame
	if not shaker then
		shaker = CreateFrame("FRAME", "ChargeMeterShaker", UIParent)
		shaker:Hide()
		shaker:SetScript("OnUpdate", function(self, elapsed)
			elapsed = self.elapsed + elapsed
			local x, y = 0, 0 -- Resets to original position if we're supposed to stop.
			if elapsed >= 0.8 then
				self:Hide()
			else
				x, y = random(-8, 8), random(-8, 8)
			end
			if WorldFrame:IsProtected() and InCombatLockdown() then
				if not shaker.fail then
					ChargeMeter:Print(L["|cffff0000Error:|r ChargeMeter cannot use shake warning if you have turned on nameplates at least once since logging in."])
					shaker.fail = true
				end
				self:Hide()
			else
				WorldFrame:ClearAllPoints()
				for i = 1, #self.originalPoints do
					local v = self.originalPoints[i]
					WorldFrame:SetPoint(v[1], v[2], v[3], v[4] + x, v[5] + y)
				end
			end
			self.elapsed = elapsed
		end)
		shaker:SetScript("OnShow", function(self)
			-- Store old worldframe positions, we need them all, people have frame modifiers for it
			if not self.originalPoints then
				self.originalPoints = {}
				for i = 1, WorldFrame:GetNumPoints() do
					tinsert(self.originalPoints, {WorldFrame:GetPoint(i)})
				end
			end
			self.elapsed = 0
		end)
		self.ShakerFrame = shaker
	end

	shaker:Show()
end
